﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MiniStm32JoeC_CoreLibrary.Components {
    public partial class SubUC_AInput : UserControl, iSubUcItems {
        MiniStm32BoardJoeC _board;
        PinInfo _pininfo;
        public SubUC_AInput(MiniStm32BoardJoeC referenceBoard, string[] para) {
            InitializeComponent();
            _board = referenceBoard;
            //0=cmd,1=loc,2=label,3=pin,4=secondPara
            txt_Label.Text = para[2];
            _pininfo = new PinInfo(para[3]);
            label_pin.Text = para[3];
            _pininfo.EnumPinMode = PinMode.IA_In_Analog;
            cb_ValueType.SelectedIndex = 0;
        }

        public string GetSetting() {
            //0=cmd,1=loc,2=label,3=pin,4=secondPara
            return $"AIN#{Location.X}:{Location.Y}#{txt_Label.Text}#{_pininfo.PinName}#{cb_ValueType.SelectedIndex}#";
        }
        public void SetSetting(string[] para) {
            switch (para[4]) {
                case "0": cb_ValueType.SelectedIndex = 0; break;
                case "1": cb_ValueType.SelectedIndex = 1; break;
                case "2": cb_ValueType.SelectedIndex = 2; break;
            }
        }
        public void Read(bool forceRead) {
            if (!chk_Active.Checked || forceRead) {
                return;
            }
            try {
                ushort counts = _board.GetPinAdc(_pininfo.PinCfg);
                switch (cb_ValueType.SelectedIndex) {
                    case 1:
                        double voltCountStep = _board.ReferenceVoltage / 4095.0;
                        double volt = voltCountStep * (double)counts;
                        txt_value.Text = $"{Math.Round(volt,2)} V";
                        break;
                    case 2:
                        double voltCountStep2 = _board.ReferenceVoltage / 4095.0;
                        double volt2 = voltCountStep2 * (double)counts;
                        txt_value.Text = $"{Math.Round(volt2*1000, 0)} mA";
                        break;
                    default:
                        txt_value.Text = counts.ToString();
                        break;
                }
                txt_value.BackColor = Color.White;
            } catch (Exception) {
                txt_value.BackColor = Color.Fuchsia;
            }
        }
        public void WriteConfigToBoard() {
            try {
                _board.SetPinLine(new string[] { _pininfo.PinCfg }, PinSetState.PinState_In_Analog);
                Read(true);
            } catch (Exception) {
                txt_value.BackColor = Color.Fuchsia;
            }
        }

        public void EditMode(bool isEditable) {
            txt_Label.Enabled = isEditable;
        }
    }
}
