﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MiniStm32JoeC_CoreLibrary.Components
{
    public partial class SubUC_DOutputSensed : UserControl, iSubUcItems {
        MiniStm32BoardJoeC _board;
        PinInfo _pininfo;
        PinInfo _pininfoSense;
        public SubUC_DOutputSensed(MiniStm32BoardJoeC referenceBoard, string[] para) {
            InitializeComponent();
            _board = referenceBoard;
            //0=cmd,1=loc,2=label,3=pin,4=secondPara
            txt_Label.Text = para[2];
            _pininfo = new PinInfo(para[3]);
            label_pin.Text = para[3];
            _pininfo.EnumPinMode = PinMode.OP_Out_PushPull;
            _pininfoSense = new PinInfo(para[4]);
            chk_Active.Text = para[4];
            _pininfoSense.EnumPinMode = PinMode.IA_In_Analog;
        }
        public string GetSetting() {
            int pinstate = 0;
            if (btn_Set_Low.BackColor != Color.Gainsboro) { pinstate = 1; }
            if (btn_Set_High.BackColor != Color.Gainsboro) { pinstate = 2; }
            if (btn_Set_HiZ.BackColor != Color.Gainsboro) { pinstate = 3; }
            //0=cmd,1=loc,2=label,3=pin,4=secondPara
            return $"SDOUT#{Location.X}:{Location.Y}#{txt_Label.Text}#{_pininfo.PinName}#{_pininfoSense.PinName}#{pinstate}";
        }
        public void SetSetting(string[] para) {
            switch (para[4]) {
                case "1":
                    btn_Set_Low.BackColor = Color.PaleGreen;
                    break;
                case "2":
                    btn_Set_High.BackColor = Color.Salmon;
                    break;
                case "3":
                    btn_Set_HiZ.BackColor = Color.Gold;
                    break;
            }
        }
        void ResetPins() {
            btn_Set_Low.BackColor = Color.Gainsboro;
            btn_Set_High.BackColor = Color.Gainsboro;
            btn_Set_HiZ.BackColor = Color.Gainsboro;
        }
        void btn_Set_Low_Click(object sender, EventArgs e)
        {
            try {
                ResetPins();
                _board.SetDigitalPin(_pininfo, false);
                Read(true);
                btn_Set_Low.BackColor = Color.PaleGreen;
            } catch (Exception) {
                btn_Set_Low.BackColor = Color.Fuchsia;
            }
        }

        void btn_Set_High_Click(object sender, EventArgs e) {
            try {
                ResetPins();
                _board.SetDigitalPin(_pininfo, true);
                Read(true);
                btn_Set_High.BackColor = Color.Salmon;
            } catch (Exception) {
                btn_Set_High.BackColor = Color.Fuchsia;
            }
        }

        void btn_Set_HiZ_Click(object sender, EventArgs e)
        {
            try {
                ResetPins();
                _board.SetPinLine(new string[] { _pininfo.PinCfg }, PinSetState.PinState_In_Float);
                Read(true);
                btn_Set_HiZ.BackColor = Color.Gold;
            } catch (Exception) {
                btn_Set_HiZ.BackColor = Color.Fuchsia;
            }
        }

        public void WriteConfigToBoard() {
            try {
                if (btn_Set_Low.BackColor != Color.Gainsboro) {
                    btn_Set_Low_Click(null, null);
                }
                if (btn_Set_High.BackColor != Color.Gainsboro) {
                    btn_Set_High_Click(null, null);
                }
                if (btn_Set_HiZ.BackColor != Color.Gainsboro) {
                    btn_Set_HiZ_Click(null, null);
                }
                _board.SetPinLine(new string[] { _pininfoSense.PinCfg }, PinSetState.PinState_In_Analog);
            } catch (Exception) {
                if (btn_Set_Low.BackColor != Color.Gainsboro) {
                    btn_Set_Low.BackColor = Color.Fuchsia;
                }
                if (btn_Set_High.BackColor != Color.Gainsboro) {
                    btn_Set_High.BackColor = Color.Fuchsia;
                }
                if (btn_Set_HiZ.BackColor != Color.Gainsboro) {
                    btn_Set_HiZ.BackColor = Color.Fuchsia;
                }
            }
        }

        public void Read(bool forceRead) {
            if (!chk_Active.Checked || forceRead) {
                return;
            }
            try {
                ushort counts = _board.GetPinAdc(_pininfoSense.PinCfg);
                txt_value.Text = counts.ToString();
                txt_value.BackColor = Color.White;
            } catch (Exception) {
                txt_value.BackColor = Color.Fuchsia;
            }
        }
        public void EditMode(bool isEditable) {
            txt_Label.Enabled = isEditable;
        }
    }
}
