﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MiniStm32JoeC_CoreLibrary.Components
{
    public partial class SubUC_I2cRW : UserControl, iSubUcItems {
        MiniStm32BoardJoeC _board;
        int collapsed = 82;
        int expanded = 0;
        int add = 0;
        int reg = 0;
        public SubUC_I2cRW(MiniStm32BoardJoeC referenceBoard, string[] para) {
            InitializeComponent();
            _board = referenceBoard;
            //0=cmd,1=loc,2=label,3=address
            txt_Label.Text = para[2];
            TryParseAddress(para[3]);
            cb_ValueType.SelectedIndex = 0;
            expanded = this.Height;
        }

        public string GetSetting() {
            //0=cmd,1=loc,2=label,3=pin(add),
            //4=secondPara (0=register,1=type)
            return $"I2CRW#{Location.X}:{Location.Y}#{txt_Label.Text}#{txt_add.Text}" +
                $"#{txt_Register.Text}|{cb_ValueType.SelectedIndex}|{txt_DataWrite.Text}#";
        }
        public void SetSetting(string[] para) {
            try {
                string[] para2 = para[4].Split('|');
                TryParseRegister(para2[0]);
                cb_ValueType.SelectedIndex = int.Parse(para2[1]);
                txt_DataWrite.Text = para2[2];
                this.Height = collapsed;
            } catch (Exception) {
                txt_Data.Text = "-";
            }
        }
        public void WriteConfigToBoard() {
            //do nothing
        }
        public void EditMode(bool isEditable) {
            if (txt_Label.Enabled != isEditable) { //changed
                if (isEditable) {
                    txt_Label.Enabled = true;
                    txt_Register.Enabled = true;
                } else {
                    txt_Label.Enabled = false;
                    txt_Register.Enabled = false;
                    this.Height = collapsed;
                }
            }
        }
        public void Read(bool forceRead) {
            //do nothing
        }

        public void doRead() {
            try {
                byte[] tx = new byte[2];
                tx[0] = (byte)((reg >> 8) & 0xff);
                tx[1] = (byte)(reg & 0xff);
                byte[] rx = _board.I2CWriteReadDirect(add, 2, tx, true);
                if (rx.Length < 5) {
                    txt_Data.Text = "-";
                    return;
                }
                //0=9B,1=2(tx cnt),2=2(rx cnt),3=rx b0,4=rx b1
                int data = rx[3] << 8 | rx[4];
                switch (cb_ValueType.SelectedIndex) {
                    case 0: txt_Data.Text = $"0x{data.ToString("X04")}"; break;
                    case 1: txt_Data.Text = $"{data}"; break;
                }
                btn_Read.BackColor = Color.Gainsboro;
            } catch (Exception) {
                btn_Read.BackColor = Color.Fuchsia;
            }
        }
        void btn_Read_Click(object sender, EventArgs e) {
            doRead();
        }
        void TryParseRegister(string register) { 
            try {
                if (register.StartsWith("0x")) {
                    reg = int.Parse(register.Remove(0, 2), System.Globalization.NumberStyles.HexNumber);
                } else {
                    reg = int.Parse(register);
                }
                txt_Register.Text = register;
                txt_Register.BackColor = Color.White;
            } catch (Exception) {
                txt_Register.BackColor = Color.Salmon;
            }
        }
        void TryParseAddress(string address) {
            try {
                if (address.StartsWith("0x")) {
                    add = int.Parse(address.Remove(0, 2), System.Globalization.NumberStyles.HexNumber);
                } else {
                    add = int.Parse(address);
                }
                txt_add.Text = address;
                txt_add.BackColor = Color.White;
            } catch (Exception) {
                txt_add.BackColor = Color.Salmon;
            }
        }

        void txt_Register_TextChanged(object sender, EventArgs e) {
            TryParseRegister(txt_Register.Text);
        }
        void txt_add_TextChanged(object sender, EventArgs e) {
            TryParseAddress(txt_add.Text);
        }

        void SubUC_I2cR_MouseDoubleClick(object sender, MouseEventArgs e) {
            if (txt_Label.Enabled) {
                if (this.Height == collapsed) {
                    this.Height = expanded;
                } else {
                    this.Height = collapsed;
                }
            }
        }
        byte[] InputDataToTxArray_16bit() {
            int value = 0;
            if (txt_DataWrite.Text.StartsWith("0x")) {
                value = int.Parse(txt_DataWrite.Text.Remove(0, 2), System.Globalization.NumberStyles.HexNumber);
            } else {
                value = int.Parse(txt_DataWrite.Text);
            }
            byte[] output = new byte[4];
            output[0] = (byte)((reg >> 8) & 0xff);
            output[1] = (byte)(reg & 0xff);
            output[2] = (byte)((value >> 8) & 0xff);
            output[3] = (byte)(value & 0xff);
            return output;
        }
        void btn_Write_Click(object sender, EventArgs e) {
            try {
                byte[] tx = null;
                switch (cb_ValueType.SelectedIndex) {
                    case 0: tx = InputDataToTxArray_16bit(); break;
                    case 1: tx = InputDataToTxArray_16bit(); break;
                }
                _board.I2CWriteDirect(add, tx);
                btn_Write.BackColor = Color.Gainsboro;
            } catch (Exception) {
                btn_Write.BackColor = Color.Fuchsia;
            }
        }
    }
}
