﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MiniStm32JoeC_CoreLibrary.Components {
    public partial class SubUC_TSens_HTU21D : UserControl, iSubUcItems {
        MiniStm32BoardJoeC _board;
        public byte i2cAddress = 128;
        public SubUC_TSens_HTU21D(MiniStm32BoardJoeC referenceBoard, string[] para) {
            InitializeComponent();
            _board = referenceBoard; 
            try {
                txt_Label.Text = para[2];
            } catch (Exception) {
                txt_Label.Text = "-";
            }
        }

        public string GetSetting() {
            return $"HTU21D#{Location.X}:{Location.Y}#{txt_Label.Text}#";
        }
        public void SetSetting(string[] para) {
            //do nothing
        }
        public float[] ReadTempHum() {
            try {
                byte[] rx = _board.I2CWriteReadDirect(i2cAddress, 3, new byte[] { 0xE3 }, true); //temp with hold
                int rawT = (rx[3] << 8) | (rx[4] & 0xFC); //14bit msb,lsb+status
                rx = _board.I2CWriteReadDirect(i2cAddress, 3, new byte[] { 0xE5 }, true); //hum with hold
                int rawH = (rx[3] << 8) | (rx[4] & 0xF0); //12bit msb,lsb+status
                //calc
                //float temp = -46.85f + (175.72f * (float)rawT / 0xffff); //datasheet
                float temp = (0.002681f * (float)rawT) - 46.85f; //lib
                //float hum = -6f + (125f * (float)rawH / 0xffff);//datasheet
                float hum = (0.001907f * (float)rawH) - 6f; //lib
                float humComp = hum + (25.0f - temp) * -0.15f; //at range 0°C..80°C, for HTU21D & SHT21 only 
                return new float[] { temp, humComp };
            } catch (Exception) {
                txt_value_Temp.BackColor = Color.Fuchsia;
                txt_value_Hum.BackColor = Color.Fuchsia;
            }
            return new float[2];
        }
        public void Read(bool forceRead) {
            if (!forceRead) {
                if (!chk_Active.Checked) {
                    return;
                }
            }
            
            try {
                float[] data = ReadTempHum();
                txt_value_Temp.Text = $"{Math.Round(data[0], 2)} °C";
                txt_value_Hum.Text = $"{Math.Round(data[1], 1)} %";
                txt_value_Temp.BackColor = Color.White;
                txt_value_Hum.BackColor = Color.White;
            } catch (Exception) {
                txt_value_Temp.BackColor = Color.Fuchsia;
                txt_value_Hum.BackColor = Color.Fuchsia;
            }
        }
        public void WriteConfigToBoard() {
            try {
                //_board.SetPinLine(new string[] { _pininfo.PinCfg }, PinSetState.PinState_In_Analog);
                Read(true);
            } catch (Exception) {
                txt_value_Temp.BackColor = Color.Fuchsia;
            }
        }
        public void EditMode(bool isEditable) {
            txt_Label.Enabled = isEditable;
        }

        void btn_read_Click(object sender, EventArgs e) {
            Read(true);
        }
    }
}
