﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using MiniStm32JoeC_CoreLibrary.Components;

namespace MiniStm32JoeC_CoreLibrary.Views
{
    public partial class UC_GUI : UserControl
    {
        public string GuiFolder = "";
        MiniStm32BoardJoeC _board;
        Control _selectedItem = null;
        public UC_GUI()
        {
            InitializeComponent();
            txt_ItemLabel.Text = "";
            GenerateGuiItems();
        }
        public void Init(MiniStm32BoardJoeC miniStm32) {
            _board = miniStm32;
            GuiFolder = Application.StartupPath + "\\GuiSetup";
            SetFolder(GuiFolder);
        }
        public void SetFolder(string path) {
            if (!Directory.Exists(path)) {
                Directory.CreateDirectory(path);
            }
            openFileDialog1.InitialDirectory = path;
            saveFileDialog1.InitialDirectory = path;
        }

        //######### new items here begin ####################################################################################
        void GenerateGuiItems() {
            //---------------------"#################################"
            cb_ItemType.Items.Add($"DOUT    |D_Output");
            cb_ItemType.Items.Add($"DIN     |D_Input");
            cb_ItemType.Items.Add($"SDOUT   |D_OutputSensed");
            cb_ItemType.Items.Add($"POWER   |D_Output+Analog in");
            cb_ItemType.Items.Add($"AIN     |Analog");
            cb_ItemType.Items.Add($"HTU21D  |I2C-TempSens");
            cb_ItemType.Items.Add($"I2CRW   |I2C-ReadWrite");
            cb_ItemType.Items.Add($"I2CW    |I2C-Write");
            cb_ItemType.Items.Add($"I2CR    |I2C-Read");
            cb_ItemType.Items.Add($"COL     |Color(Rgb Led)");
            cb_ItemType.SelectedIndex = 0;
        }
        bool AddGuiItem(string[] para, bool withOffset = false) {
            try {
                Control item = null;
                switch (para[0]) {
                    case "DOUT":    item = new SubUC_DOutput(_board, para); break;
                    case "DIN":     item = new SubUC_DInput(_board, para); break;
                    case "SDOUT":   item = new SubUC_DOutputSensed(_board, para); break;
                    case "POWER":   item = new SubUC_PowerSupply(_board, para); break;
                    case "AIN":     item = new SubUC_AInput(_board, para); break;
                    case "HTU21D":  item = new SubUC_TSens_HTU21D(_board, para); break;
                    case "I2CRW":   item = new SubUC_I2cRW(_board, para); break;
                    case "I2CW":    item = new SubUC_I2cW(_board, para); break;
                    case "I2CR":    item = new SubUC_I2cR(_board, para); break;
                    case "COL":     item = new SubUC_Color(_board, para); break;
                }
        //######### new items here end ####################################################################################

                if (item == null) {
                    return false;
                }
                (item as iSubUcItems).SetSetting(para);
                panel_items.Controls.Add(item);
                Control C = panel_items.Controls[panel_items.Controls.Count - 1];
                string[] splits = para[1].Split(':');
                C.Location = new Point(int.Parse(splits[0]), int.Parse(splits[1]));
                if (withOffset) {
                    C.Location = new Point(int.Parse(splits[0]) + 10, int.Parse(splits[1]) + 10);
                    item.BringToFront();
                }
                C.MouseEnter += C_MouseEnter;
                C.MouseLeave += C_MouseLeave;
                C.MouseMove += C_MouseMove;
                C.MouseDown += C_MouseDown;
                (C as iSubUcItems).EditMode(chk_EditMode.Checked);
                groupBox_add.Visible = false;
                return true;
            } catch (Exception ex) {
                MessageBox.Show("AddGuiItem error: " + ex.Message);
            }
            return false;
        }

        void btn_ItemAdd_Click(object sender, EventArgs e) {
            string cmd = cb_ItemType.SelectedItem.ToString().Split('|')[0].TrimEnd();
            //0=cmd,1=loc,2=label,3=pin,4=secondPara
            string[] para = new string[] {
                cmd,
                $"{100}:{(panel_items.Height/2)}",
                txt_ItemLabel.Text,
                txt_ItemPin.Text,
                txt_secondParameter.Text };
            AddGuiItem(para);
            panel_items.Controls.SetChildIndex(panel_items.Controls[panel_items.Controls.Count - 1], 0);
        }

        void tbtn_LoadGui_Click(object sender, EventArgs e) {
            if (DialogResult.OK != openFileDialog1.ShowDialog()) {
                return;
            }
            LoadGuiFile(openFileDialog1.FileName, false);
        }

        void tbtn_SaveGui_Click(object sender, EventArgs e) {
            if (saveFileDialog1.ShowDialog() != DialogResult.OK) {
                return;
            }
            string filename = saveFileDialog1.FileName;
            if (!filename.EndsWith(".txt")) {
                filename += ".txt";
            }
            try {
                StreamWriter txt = new StreamWriter(filename);
                txt.WriteLine("# all lines starting with '#' will be ignored (comments)");
                txt.WriteLine("########################");
                txt.WriteLine($"READTIMER#{chk_read_timer.Checked}#");

                foreach (var item in panel_items.Controls) {
                    txt.WriteLine((item as iSubUcItems).GetSetting());
                }
                txt.Close();
                MessageBox.Show("Saved: " + filename, "Saved...");
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "Not Saved...");
            }
        }
        public bool LoadGuiFile(string path, bool filenameOnly) { 
            try {
                if (filenameOnly) {
                    path = GuiFolder + "\\" + path;
                    if (!File.Exists(path)) {
                        MessageBox.Show("Gui file not found:\r\n" + path, "Load Gui File failed");
                        return false;
                    }
                }
                StreamReader txt = new StreamReader(path);
                string[] inhalt = txt.ReadToEnd().Split('\n');
                txt.Close();

                panel_items.Controls.Clear();
                bool readTimerEnabled = false;
                StringBuilder sb_unknown = new StringBuilder();
                for (int i = 0; i < inhalt.Length; i++) {
                    if (inhalt[i].StartsWith("#")) {
                        continue; //comment
                    }
                    string[] splits = inhalt[i].Split('#');

                    if (splits.Length < 3) {
                        continue; //not enough items
                    }
                    bool added = AddGuiItem(splits);
                    if (!added) {
                        if (splits[0] == "READTIMER") {
                            readTimerEnabled = bool.Parse(splits[1]);
                        } else { 
                            sb_unknown.AppendLine($"Unknown tag '{splits[0]}' from: {inhalt[i].TrimEnd()}");
                        }
                    }
                }
                //foreach (var item in panel_items.Controls) {
                //    (item as iSubUcItems).WriteConfigToBoard();
                //}
                if (sb_unknown.Length != 0) {
                    MessageBox.Show("Some items could not be loaded:\r\n" + sb_unknown.ToString(), "Load Gui File failed");
                    return false;
                }
                chk_read_timer.Checked = readTimerEnabled;
                return true;
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "Not Loaded...");
            }
            return false;
        }

        void num_read_Timer_ValueChanged(object sender, EventArgs e) {
            timer_read.Interval = (int)num_read_Timer.Value;
        }
        void chk_read_timer_CheckedChanged(object sender, EventArgs e) {
            timer_read.Enabled = chk_read_timer.Checked;
            btn_read.BackColor = (chk_read_timer.Checked) ? Color.Gold : Color.Gainsboro;
        }
        bool isReading = false;
        void timer_read_Tick(object sender, EventArgs e) {
            if (isReading) {
                return;
            }
            isReading = true;
            foreach (iSubUcItems item in panel_items.Controls) {
                item.Read(false);
            }
            isReading = false;
        }



        Point _MouseDown = new Point();
        Point _InitialLocation = new Point();
        void C_MouseDown(object sender, MouseEventArgs e) {
            if (e.Button != MouseButtons.Left) {
                return;
            }
            Control C = sender as Control;
            _InitialLocation = C.Location;
            _MouseDown = Cursor.Position;
        }

        void C_MouseMove(object sender, MouseEventArgs e) {
            if (e.Button != MouseButtons.Left) {
                return;
            }
            Control C = sender as Control;
            if (C.BackColor != Color.LimeGreen) {
                return;
            }
            C.Location = new Point(
                Cursor.Position.X - _MouseDown.X + _InitialLocation.X, 
                Cursor.Position.Y - _MouseDown.Y + _InitialLocation.Y); 
        }
        void C_MouseLeave(object sender, EventArgs e) {
            (sender as Control).BackColor = Color.Silver;
        }

        void C_MouseEnter(object sender, EventArgs e) {
            if (!chk_EditMode.Checked) {
                return;
            }
            _selectedItem = sender as Control;
            _selectedItem.BackColor = Color.LimeGreen;
        }

        void btn_gui_addWindow_Click(object sender, EventArgs e) {
            groupBox_add.Visible = !groupBox_add.Visible;
        }

        void btn_ItemCancel_Click(object sender, EventArgs e) {
            groupBox_add.Visible = false;
        }

        void tbtn_ReadAll_Click(object sender, EventArgs e) {
            foreach (iSubUcItems item in panel_items.Controls) {
                item.Read(false);
            }
        }

        void tbtn_Configure_Click(object sender, EventArgs e) {
            foreach (iSubUcItems item in panel_items.Controls) {
                item.WriteConfigToBoard();
            }
        }

        void btn_read_Click(object sender, EventArgs e) {
            foreach (iSubUcItems item in panel_items.Controls) {
                item.Read(false);
            }
        }

        void chk_EditMode_Click(object sender, EventArgs e) {
            foreach (iSubUcItems item in panel_items.Controls) {
                item.EditMode(chk_EditMode.Checked);
            }
        }

        void tbtn_ClearAll_Click(object sender, EventArgs e) {
            if (DialogResult.Yes == MessageBox.Show("Clear all items?","Gui Items",MessageBoxButtons.YesNo)) {
                panel_items.Controls.Clear();
            }
        }

        void tbtn_DeleteItem_Click(object sender, EventArgs e) {
            if (_selectedItem == null) {
                return;
            }
            panel_items.Controls.Remove(_selectedItem);
            _selectedItem = null;
        }
        void tbtn_ItemToTop_Click(object sender, EventArgs e) {
            if (_selectedItem == null) {
                return;
            }
            panel_items.Controls.SetChildIndex(_selectedItem, 0);
        }
        void tbtn_ItemClone_Click(object sender, EventArgs e) {
            if (_selectedItem == null) {
                return;
            }
            string[] settings = (_selectedItem as iSubUcItems).GetSetting().Split('#');
            AddGuiItem(settings, true);
        }

    }
}
