﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MiniStm32JoeC_CoreLibrary.Views {
    public partial class UC_UART : UserControl {
        MiniStm32BoardJoeC _board;
        int address = 0;
        int WaitForBusyTimeout = 1;
        public UC_UART() {
            InitializeComponent();
            CB_RS232_baud.SelectedIndex = 2;
        }
        public void Init(MiniStm32BoardJoeC miniStm32, int port) {
            _board = miniStm32;
            if (port == 1) {
                _board.Event_RxUart1 += AddToLog;
            }
            if (port == 3) {
                _board.Event_RxUart3 += AddToLog;
            }

            label_Port.Text = "UART " + port.ToString();
            address = port;
        }

        //bool _closing = false;
        //public void Close() {
        //    _closing = true;
        //}
        void WaitforBusy(int timeoutInSeconds) {
            DateTime dtTimeout = DateTime.Now.AddSeconds(timeoutInSeconds);
            while (_board.IsBusy) {
                if (DateTime.Now.Ticks > dtTimeout.Ticks) {
                    _board.IsBusy = false;
                    throw new Exception($"WaitforBusy timeout after '{timeoutInSeconds}'");
                }
                Application.DoEvents();
                System.Threading.Thread.Sleep(1);
            }
        }

        void btnUartSetupBaud_Click(object sender, EventArgs e) {
            try {
                UInt32 baud = UInt32.Parse(txt_rs232_baud.Text);
                _board.UartInit(address, baud);
            } catch (Exception ex) {
                MessageBox.Show(ex.Message);
            }
        }

        void txt_send_txt_KeyDown(object sender, KeyEventArgs e) {
            if (e.KeyCode == Keys.Enter) {
                try {
                    if (_board.IsBusy) {
                        return;
                    }
                    _board.IsBusy = true;
                    string tx = txt_send_txt.Text;
                    if (chk_add_EndBytes.Checked) {
                        tx += (char)num_endByte.Value;
                    }
                    _board.UartWrite(address, tx);
                } catch (Exception ex) {
                    MessageBox.Show(ex.Message);
                }
                _board.IsBusy = false;
            }
        }

        void txt_send_bytes_KeyDown(object sender, KeyEventArgs e) {
            if (e.KeyCode != Keys.Enter) {
                return;
            }
            try {
                if (_board.IsBusy) {
                    return;
                }
                _board.IsBusy = true;
                List<byte> tx = new List<byte>();
                string[] splits = txt_send_bytes.Text.Split(' ');
                byte B = 0;
                for (int i = 0; i < splits.Length; i++) {
                    if (byte.TryParse(splits[i],out B)) {
                        tx.Add(B);
                    }
                }
                if (chk_add_EndBytes.Checked) {
                    tx.Add((byte)num_endByte.Value);
                }
                _board.UartWrite(address, tx.ToArray());
            } catch (Exception ex) {
                MessageBox.Show(ex.Message);
            }
            _board.IsBusy = false;
        }

        void btn_log_Clear_Click(object sender, EventArgs e) {
            txt_rx_log.Text = "";
        }

        void chk_setup_CheckedChanged(object sender, EventArgs e) {
            group_setup.Visible = chk_setup.Checked;
        }

        void CB_RS232_baud_SelectedIndexChanged(object sender, EventArgs e) {
            txt_rs232_baud.Text = CB_RS232_baud.SelectedItem.ToString();
        }
        public void AddToLog(string info) {
            txt_rx_log.AppendText(info.Replace("\r","\r\n"));
            txt_rx_log.ScrollToCaret();
        }

        void timer_Poll_Tick(object sender, EventArgs e) {
            try {
                //if (_board.IsBusy) {
                //    return;
                //}
                ReadBytes(10, 0);
            } catch (Exception ex) {
                AddToLog($"EX:{ex}");
            }
        }

        void txt_WriteRead_txt_KeyDown(object sender, KeyEventArgs e) {
            if (e.KeyCode != Keys.Return) {
                return;
            }
            try {
                WaitforBusy(WaitForBusyTimeout);
                _board.IsBusy = true;
                string tx = txt_WriteRead_txt.Text;
                if (chk_add_EndBytes.Checked) {
                    tx += (char)num_endByte.Value;
                }
                string rx = _board.UartWriteRead(address, tx, (int)num_WriteRead_txt_NrOfBytes.Value, 3);
                AddToLog(rx);
            } catch (Exception ex) {
                MessageBox.Show(ex.Message);
            }
            _board.IsBusy = false;
        }

        void txt_WriteRead_byte_KeyDown(object sender, KeyEventArgs e) {
            if (e.KeyCode != Keys.Return) {
                return;
            }
            try {
                WaitforBusy(WaitForBusyTimeout);
                _board.IsBusy = true;
                List<byte> tx = new List<byte>();
                string[] splits = txt_WriteRead_byte.Text.Split(' ');
                byte B = 0;
                txt_rx_log.AppendText("\r\nWriteRead: ");
                for (int i = 0; i < splits.Length; i++) {
                    if (byte.TryParse(splits[i], out B)) {
                        tx.Add(B);
                        txt_rx_log.AppendText(B.ToString() + " ");
                    }
                }
                if (chk_add_EndBytes.Checked) {
                    tx.Add((byte)num_endByte.Value);
                }
                txt_rx_log.AppendText(" -> ");
                byte[] rx = _board.UartWriteRead(address, tx.ToArray(), (int)num_WriteRead_byte_NrOfBytes.Value, 3);
                for (int j = 0; j < rx.Length; j++) {
                    txt_rx_log.AppendText(rx[j].ToString() + " ");
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message);
            }
            _board.IsBusy = false;
        }

        void chk_uart_poll_CheckedChanged(object sender, EventArgs e) {
            timer_Poll.Enabled = chk_uart_poll.Checked;
            if (chk_uart_poll.Checked) {
                timer_Poll.Interval = (int)num_uart_poll.Value;
                timer_Poll.Enabled = true;
            } else {
                timer_Poll.Enabled = false;
            }
        }
        int ReadBytes(int NrOfBytes, int timeout) {
            try {
                if (_board.IsBusy) {
                    return 0;
                }
                _board.IsBusy = true;
                byte[] rx = _board.UartRead(address, NrOfBytes, timeout);
                _board.IsBusy = false;
                if (rx.Length == 0) {
                    return 0;
                }
                int len = rx.Length;
                //if (len > rx.Length) {
                //    len = rx.Length;
                //}
                for (int i = 0; i < len; i++) {
                    if (rad_mode_byte.Checked) {
                        txt_rx_log.AppendText(rx[i].ToString() + " ");
                    }
                    if (rad_mode_Txt.Checked) {
                        if (rx[i] < 32) { //non printable
                            txt_rx_log.Text += "-";
                        } else {
                            char c = (char)rx[i];
                            txt_rx_log.Text += c.ToString();
                        }
                    }
                }
                return len;
            } catch (Exception ex) {
                AddToLog(ex.Message);
            }
            _board.IsBusy = false;
            return 0;
        }

        void btn_uart_readBytes_Click(object sender, EventArgs e) {
            try {
                WaitforBusy(WaitForBusyTimeout);
                //_board.IsBusy = true;
                txt_rx_log.AppendText($"ReadBytes({num_uart_ReadBytes.Value}) -> ");
                int len = ReadBytes((int)num_uart_ReadBytes.Value, 3);
                if (len == (int)num_uart_ReadBytes.Value) {
                    txt_rx_log.AppendText("\r\n");
                } else {
                    txt_rx_log.AppendText($"[Missing '{(int)num_uart_ReadBytes.Value - len}' Bytes]\r\n");
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message);
            }
            //_board.IsBusy = false;
        }
    }
}
